from blueness import module

from bluer_options import string
from bluer_options.host import is_rpi
from bluer_objects import file, objects
from bluer_objects.host import shell

from bluer_agent import NAME
from bluer_agent.audio import consts
from bluer_agent.logger import logger


NAME = module.name(__file__, NAME)


def record(
    object_name: str,
    filename: str = "audio.wav",
    length: int = 30,  # in seconds
    rate: int = 48000,
    channels: int = 1,
    crop_silence: bool = True,
) -> bool:
    full_filename = objects.path_of(
        object_name=object_name,
        filename=filename,
    )

    logger.info(
        "{}.record: {}/{} @ max: {}, sample rate: {}, {} channel(s){} ... (^C to end)".format(
            NAME,
            object_name,
            filename,
            string.pretty_duration(length),
            rate,
            channels,
            ", crop silence" if crop_silence else "",
        )
    )

    # input device:
    # - On Raspberry Pi with mic as default: don't specify device
    # - If you need a specific ALSA device: add `-t alsa hw:1,0` after rec
    if not shell(
        [
            "rec",
            "-V1",
            f'-r "{rate}"',
            f'-c "{channels}"',
            full_filename,
            f"trim 0 {length}",
        ]
        + (
            [
                "silence",
                f'1 "{consts.BLUER_AGENT_AUDIO_LISTEN_START_HOLD}" "{consts.BLUER_AGENT_AUDIO_LISTEN_START_THRESHOLD}"',
                f'1 "{consts.BLUER_AGENT_AUDIO_LISTEN_STOP_HOLD}" "{consts.BLUER_AGENT_AUDIO_LISTEN_STOP_THRESHOLD}"',
            ]
            if crop_silence
            else []
        )
    ):
        return False

    if not file.exists(full_filename):
        logger.error(f"{full_filename} was not created.")
        return False

    logger.info(
        "audio size: {}".format(
            string.pretty_bytes(file.size(full_filename)),
        )
    )
    return True



from blueness import module
from bluer_objects import file, objects
from bluer_objects.metadata import post_to_object

from bluer_agent import NAME
from bluer_agent.logger import logger


NAME = module.name(__file__, NAME)


def post_process(
    object_name: str,
    filename: str,
    language: str = "fa",
) -> bool:
    logger.info(
        "{}.post_process({}/{}) [{}]".format(
            NAME,
            object_name,
            file.name_and_extension(filename),
            language,
        )
    )

    success, content = file.load_json(
        objects.path_of(
            object_name=object_name,
            filename="transcript.json",
        ),
        default={},
    )
    if not success:
        return success

    if not isinstance(content, dict):
        logger.error("content is not a dict")
        return False

    text = content.get("text", "")

    logger.info(text)

    return post_to_object(
        object_name,
        file.name(filename),
        text,
    )



from bluer_options.env import load_config, load_env, get_env

load_env(__name__)
load_config(__name__)

BLUER_AGENT_MACHINE_USER_NAME = get_env("BLUER_AGENT_MACHINE_USER_NAME")

BLUER_AGENT_API_KEY = get_env("BLUER_AGENT_API_KEY")

BLUER_AGENT_TRANSCRIPTION_ENDPOINT = get_env("BLUER_AGENT_TRANSCRIPTION_ENDPOINT")
BLUER_AGENT_TRANSCRIPTION_TEST_OBJECT = get_env("BLUER_AGENT_TRANSCRIPTION_TEST_OBJECT")

BLUER_AGENT_CHAT_ENDPOINT = get_env("BLUER_AGENT_CHAT_ENDPOINT")
BLUER_AGENT_CHAT_TEST_OBJECT = get_env("BLUER_AGENT_CHAT_TEST_OBJECT")

BLUER_AGENT_TRANSCRIPTION_RETRIAL = get_env("BLUER_AGENT_TRANSCRIPTION_RETRIAL", 5)

BLUER_AGENT_AUDIO_TEST_OBJECT = get_env("BLUER_AGENT_AUDIO_TEST_OBJECT")



from blueness import module
from bluer_objects import file, objects
from bluer_objects.metadata import post_to_object

from bluer_agent import NAME
from bluer_agent.logger import logger


NAME = module.name(__file__, NAME)


def post_process(
    object_name: str,
) -> bool:
    logger.info(f"{NAME}.post_process({object_name})")

    success, content = file.load_json(
        objects.path_of(
            object_name=object_name,
            filename="chat.json",
        ),
        default={},
    )
    if not success:
        return success

    if not isinstance(content, dict):
        logger.error("content is not a dict")
        return False

    if "choices" not in content:
        logger.error("choices not in content")
        return False
    if len(content["choices"]) == 0:
        logger.error("choices is empty")
        return False
    if len(content["choices"]) > 1:
        logger.warning(
            "{} choice(s), will use the first one.".format(content["choices"])
        )

    text = content["choices"][0].get("message", {}).get("content", "")
    logger.info(text)

    return post_to_object(
        object_name,
        "chat",
        text,
    )



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_agent import NAME
from bluer_agent.audio.play import play
from bluer_agent.audio.record import record
from bluer_agent.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="play | record",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--filename",
    type=str,
    default="audio.wav",
)
parser.add_argument(
    "--length",
    type=int,
    default=30,
    help="in seconds",
)
parser.add_argument(
    "--crop_silence",
    type=int,
    default=1,
    help="0 | 1",
)
parser.add_argument(
    "--rate",
    type=int,
    default=48000,
)
parser.add_argument(
    "--channels",
    type=int,
    default=1,
)
args = parser.parse_args()

success = False
if args.task == "play":
    success = play(
        object_name=args.object_name,
        filename=args.filename,
    )
elif args.task == "record":
    success = record(
        object_name=args.object_name,
        filename=args.filename,
        length=args.length,
        rate=args.rate,
        channels=args.channels,
        crop_silence=args.crop_silence == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_validate(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra(
        "upload,verbose",
        mono=mono,
    )

    return show_usage(
        [
            "@agent",
            "chat",
            "validate",
            f"[{options}]",
            "[-|<object-name>]",
        ],
        "validate chatting.",
        mono=mono,
    )


help_functions = {
    "validate": help_validate,
}



from typing import List

from bluer_options.terminal import show_usage, xtra

from bluer_agent.help.audio import record_args


def help(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("filename=<filename.wav>,install", mono=mono),
            "language=en|fa",
        ]
    )

    record_options = xtra("download,~record,~play,upload", mono=mono)

    return show_usage(
        [
            "@agent",
            "transcribe",
            f"[{options}]",
            "[-|<object-name>]",
            f"[{record_options}]",
        ]
        + record_args,
        "validate transcription.",
        mono=mono,
    )



from bluer_agent.README.items import items
from bluer_agent.README import aliases, audio, chat, transcription

docs = (
    [
        {
            "path": "../..",
            "items": items,
        },
        {
            "path": "../docs",
        },
    ]
    + aliases.docs
    + audio.docs
    + chat.docs
    + transcription.docs
)



from bluer_objects import README

items = README.Items(
    [
        {
            "name": "chat",
            "marquee": "https://github.com/kamangir/assets2/raw/main/bluer-agent/icon.jpg?raw=true",
            "url": "./bluer_agent/docs/chat",
        },
        {
            "name": "transcription",
            "marquee": "https://github.com/kamangir/assets2/raw/main/bluer-agent/icon.jpg?raw=true",
            "url": "./bluer_agent/docs/transcription",
        },
    ]
)
